// ANNForecastAppAgent.m                                        

#import "ANNForecastAppAgent.h"

// static to avoid 'duplicate name' error (the function is defined also 
// in the .m files of other kinds of agent; the content anyway is different)
static float chooseBuySellSwitch(id forecastingAgent, float asymmetricBuySellProb,
                                 float aNNInactivityRange)
{
  float buySellSwitch;

   buySellSwitch=-1.; // doesn't act

   // today is day t; we are now following the price estimate for day 
   // t-1+nAheadForecasting made in t (today) with t-1 data
   if([forecastingAgent getCurrentEstimateIndex]>1+aNNInactivityRange) // buy
                                  buySellSwitch=asymmetricBuySellProb;
   if([forecastingAgent getCurrentEstimateIndex]<1-aNNInactivityRange) // sell
                                  buySellSwitch=1-asymmetricBuySellProb;

  return buySellSwitch;
}

// static anyway, see above
static float checkToAvoidNonsense(id forecastingAgent, id theBook, 
                                  int printing)
{
  float index, currentPrice, meanPrice;
  int check;

  check=1;
  index=[forecastingAgent getCurrentEstimateIndex];
  currentPrice=[theBook getPrice];  // in day t
  meanPrice=[theBook getMeanPrice]; // of day t-1

  // the current price is greater than the posivite forecasting, so it would
  // be a nonsense to buy
  if(index>1 && currentPrice>meanPrice*index) check=-1;

  // the current price is less than the negative forecasting, so it would
  // be a nonsense to sell
  if(index<1 && currentPrice<meanPrice*index) check=-1;

  if(printing==5)printf("cP %f mP %f in %f check %d\n", 
                        currentPrice, meanPrice, index, check);

  return check;

  // NB this check is done before asking ruleMaster for a price; the price returned
  // by ruleMater can produce one of the nonsense checked below, but this fact is consistent
  // with the non completely deterministic behavior of our agents; in other term, we state
  // that their is no nonsense in following forecasting; then we have a buySellSwitch
  // allowing the agent to do also an action opposite to that suggested by the
  // forecast; finally we have also the possibility of acting with a random shift applyed to
  // the price in a measure producing ex post the nonsense that we avoid ex ante

}


@implementation ANNForecastAppAgent
 

  // our ruleMaster
- setRuleMaster: r
{
  ruleMaster=r;
  return self;
}

- setForecastingAgent: f
{
  forecastingAgent=f;
  return self;
}

- setMyUniformDblRand: u
{
  myUniformDblRand = u;
  return self;
}

- setANNInactivityRange: (float) r 
                  andANNForecastAppAgentActDailyProb: (float) p
{
  aNNInactivityRange = r;
  aNNForecastAppAgentActDailyProb = p;

  return self;
}

  // Acting before opening
- act0
{
   float lastPrice;
   int i;

     // clearing executedPrices
        executedPriceCount=0;
        for (i=0;i<maxOrderQuantity;i++) [executedPrices R:i C:0 setFrom:0];

   actingBeforeOpening=0;

   // deciding if this an operative day for this agent
   if (aNNForecastAppAgentActDailyProb > (float)
       [myUniformDblRand getDoubleWithMin: 0.0 withMax: 1.0])
    // if yes
    { 
       // applying the forecast (if the forecast is not used, the result is -1)
       buySellSwitch=chooseBuySellSwitch(forecastingAgent, asymmetricBuySellProb,
                                         aNNInactivityRange);

       lastPrice=[theBook getPrice];

       // Asking the ruleMaster for a position (if the price is > 0 we are
       // buyer; if it is <0 we are seller; if it is =0 we do nothing
       // the ruleMaster uses in this case the probability of operating
       // before opening
       if (buySellSwitch != -1.) // buySellSwitch indicates that the agent
	                         // has to operate
          {if(checkToAvoidNonsense(forecastingAgent, theBook, printing) != -1)
            price=[ruleMaster getPriceBeforeOpening: lastPrice
                                  withBuySellP: buySellSwitch];
	   else price=0;
           if (price!=0) actingBeforeOpening=1;
          }
       else {price=0;actingBeforeOpening=1;}// so 'act' don't operate in this day

     }
     else
     // if no
     {
       price=0;
       actingBeforeOpening=1; // so 'act' don't operate in this day
     }

   // how many orders?
   if (maxOrderQuantity==1) iMax=1;
   else iMax=[uniformIntRand getIntegerWithMin: 1 withMax: maxOrderQuantity];
   for (i=1;i<=iMax;i++)
   // if price==0 theBook is not recording it
   [theBook setOrderBeforeOpeningFromAgent: number atPrice: price];

   return self;
}

  // acting in the market
- act1
{
   float lastPrice;
   int i;

   // the agent has operated before opening
   if (actingBeforeOpening==1) return self;

   // applying the forecast (the -1 case appears in opening phase and
   //                        eliminate the act phase, setting 
   //                        actingBeforeOpening=1)
   buySellSwitch=chooseBuySellSwitch(forecastingAgent, asymmetricBuySellProb,
                                     aNNInactivityRange);

   lastPrice=[theBook getPrice];

   // if we are here, buySellSwitch!=-1.
   if(checkToAvoidNonsense(forecastingAgent, theBook, printing) != -1)
   price=[ruleMaster getPrice: lastPrice
                           withBuySellP: buySellSwitch];
   else price=0;

   // how many orders?
   if (maxOrderQuantity==1) iMax=1;
    else iMax=[uniformIntRand getIntegerWithMin: 1 withMax: maxOrderQuantity];
   for (i=1;i<=iMax;i++)
   [theBook setOrderFromAgent: number atPrice: price];

   return self;
}

@end






