// Book.m                                        

#import "Book.h"
#import "BasicSumAgent.h" // this is placed here to avoid a circular call

  // functions

void message(id <Index> agentArrayIndex, int n, float p)
{
   BasicSumAgent * anAgent;

   [agentArrayIndex setOffset: n-1];
   anAgent=[agentArrayIndex get];
   [anAgent setConfirmationOfExecutedPrice: p];
   return;
}

void setLocal(Matrix2 * loc, float p)
{
   int i;
   if (p==0)return;
   for (i=[loc getRows]-2;i>=0;i--)[loc R:i+1 C:0 setFrom: [loc R:i C:0]];
   [loc R:0 C:0 setFrom: p];
   return;
}

int getLocal(Matrix2 * loc)
{
   int i, tot;
   tot=0;
   for (i=0;i<[loc getRows];i++)
      {
      if([loc R:i C:0]>0)tot++;
      if([loc R:i C:0]<0)tot--;
      }
   return tot;
}

@implementation Book

- setAgentArrayIndex: i
{
  agentArrayIndex = i;
  return self;
}

- setAgentNumber: (int) n
{
  agentNumber=n;
  return self;
}

- setMaxOrderQuantity: (int) m
{
  maxOrderQuantity=m;
  return self;
}
- setProbMarketOrder: (float) mo
{
  probMarketOrder=mo;
  return self;
}
- setMeanPriceHistoryLength: (int) l
{
  meanPriceHistoryLength=l;
  return self;
}

- setLocalHistoryLength: (int) l
{
  localHistoryLength=l;
  return self;
}

- setPrinting: (int) p
{
  printing=p;
  return self;
}

- createEnd
{
   int i;
   [super createEnd];

   executedPrice=1; // this is the starting price; it seems to be
                    // not relevant at all for the behavior of the model
   meanPrice=1;
   previousClosingPrice=executedPrice;
   currentMeanPrice=0;
   count=0;


   // the book works on the basis of two matrixes containing sell
   // order in increasing order or buy order in decreasing order
   // (in col 1 we have the orders; in col 2 the number of the agent
   // placing the order)

   // if an order obtains an immediate matching, it is not filed

   // the worse situation is that of having all the order on one side of
   // the market and all the agents ordering; so the rows of the two
   // matrixes must be equal to the number of the agents
    
   sellOrderStorehouse=[Matrix2 createBegin: [self getZone]];
   [sellOrderStorehouse setDimensionRows: agentNumber*maxOrderQuantity
                                Cols: 2 Code: 1];
   sellOrderStorehouse=[sellOrderStorehouse createEnd];

   buyOrderStorehouse=[Matrix2 createBegin: [self getZone]];
   [buyOrderStorehouse  setDimensionRows: agentNumber*maxOrderQuantity
                                Cols: 2 Code: 2];
   buyOrderStorehouse=[buyOrderStorehouse   createEnd];

   meanPriceHistory=[Matrix2 createBegin: [self getZone]];
   [meanPriceHistory setDimensionRows: meanPriceHistoryLength
                                  Cols: 1 Code: 3];
   meanPriceHistory=[meanPriceHistory createEnd];
   // mean prices will be stored by rows; now we fill all the r. with
   // the starting mean price
   for (i=0;i<=meanPriceHistoryLength-1;i++)
       [meanPriceHistory R:i C:0 setFrom: meanPrice];

   localHistory=[Matrix2 createBegin: [self getZone]];
   [localHistory setDimensionRows: localHistoryLength
                                  Cols: 1 Code: 4];
   localHistory=[localHistory createEnd];
   // local actions will be stored by rows; we fill all the r. with
   // 0, i.e. 'no action', automatically, as a byproduct of the
   // setDimensionRows:Cols: method

   return self;
}

  // at the end of each day
- setMeanPrice{
  if (count>0) meanPrice=currentMeanPrice/count; // otherwise we keep
                                                 // previous value
  return self;
}

- setNAheadForecasting: (int) na
{
  nAheadForecasting = na;
  return self;
}

  // at the beginning of each day
- setClean{
  int i;
  sellOrderNumber=0; 
  buyOrderNumber=0;

  // meanPriceHistory in row 0 contains the t-1 meanPrice;
  //                  in row 1 contains the t-2 meanPrice;
  //                  etc.
  for (i=meanPriceHistoryLength-2;i>=0;i--)
                    [meanPriceHistory R:i+1 C:0 setFrom:
                                      [meanPriceHistory R:i C:0]];
  [meanPriceHistory R:0 C:0 setFrom: meanPrice];
               
  previousClosingPrice=executedPrice; // the last one of 'yesterday'
  currentMeanPrice=0;
  count=0;
  return self;
}

  // receiving an order before opening from an agent
- setOrderBeforeOpeningFromAgent: (int) n atPrice: (float) p
{
   int number;

   number = n;
   price = p;

   if(printing==1)
   printf(
  "The book received a before opening order from agent #%3d at price %7.4f\n",
          number, price);

   // local history
   setLocal(localHistory, price);

   // if price==0 no action required, but sending a 0.0 message to the agent
   if(price==0) message(agentArrayIndex, number, 0.0);

   // the agent is selling at min price '-price'
   if(price<0)      {
                     message(agentArrayIndex, number, 0.0);
                     // filing the sell order, in increasing order
                     sellOrderNumber++;
                     [sellOrderStorehouse fileIncreasingP: -price
                                                 andN: (float) number
                                usingAsNumberOfRows: sellOrderNumber];
                     if(printing==1)
                     [sellOrderStorehouse printNRows: sellOrderNumber];
                     sellOrderNumberBeforeOpening=sellOrderNumber;

                    }

   // the agent is buying at max price 'price'
   if(price>0)      {
                     message(agentArrayIndex, number, 0.0);
                     // filing the buy order, in decreasing order
                     buyOrderNumber++;
                     [buyOrderStorehouse fileDecreasingP: price
                                                andN: (float) number
                                usingAsNumberOfRows: buyOrderNumber];
                     if(printing==1)
                     [buyOrderStorehouse printNRows: buyOrderNumber];
                     buyOrderNumberBeforeOpening=buyOrderNumber;
                     }

   return self;
}

  // receiving an order when the market is open
- setOrderFromAgent: (int) n atPrice: (float) p
{
   int number;

   number = n;
   price = p;

   if(printing==1)
   printf("The book received an order from agent #%3d at price %7.4f\n",
          number, price);

   // local history
   setLocal(localHistory, price);

   // if price==0 no action required, but sending a 0.0 message to the agent
   if(price==0) message(agentArrayIndex, number, 0.0);

   //market orders   
   if (probMarketOrder <(float) [uniformDblRand getDoubleWithMin: 0.0 withMax: 1.0])
   { 
// the agent is selling at min price '-price'
   if(price<0) {if (buyOrderNumber>0 &&
                  [buyOrderStorehouse R: 0 C: 0] >= -price)
                    {executedPrice=[buyOrderStorehouse R: 0 C: 0];
                     currentMeanPrice+=executedPrice;
                     count++;
                     message(agentArrayIndex, number, -executedPrice);
                     message(agentArrayIndex,(int)[buyOrderStorehouse R: 0 C: 1],
                                                       executedPrice);
                     buyOrderNumber--;
                     [buyOrderStorehouse shiftRowsDown: buyOrderNumber];
                     if(printing==1)
                     [buyOrderStorehouse printNRows: buyOrderNumber];
                    }
                   
               else {
                     message(agentArrayIndex, number, 0.0);
                     // filing the sell order, in increasing order
                     sellOrderNumber++;
                     [sellOrderStorehouse fileIncreasingP: -price
                                                 andN: (float) number
                                usingAsNumberOfRows: sellOrderNumber];
                     if(printing==1)
                     [sellOrderStorehouse printNRows: sellOrderNumber];
                    }
               }

   // the agent is buying at max price 'price'
   if(price>0) {if (sellOrderNumber>0 &&
                  [sellOrderStorehouse R: 0 C: 0] <= price)
                    {executedPrice=[sellOrderStorehouse R: 0 C: 0];
                     currentMeanPrice+=executedPrice;
                     count++;
                     message(agentArrayIndex, number, executedPrice);
                     message(agentArrayIndex,(int)[sellOrderStorehouse R: 0 C: 1],
                                                      -executedPrice);
                     sellOrderNumber--;
                     [sellOrderStorehouse shiftRowsDown: sellOrderNumber];
                     if(printing==1)
                     [sellOrderStorehouse printNRows: sellOrderNumber];
                    }
                   
               else {
                     message(agentArrayIndex, number, 0.0);
                     // filing the buy order, in decreasing order
                     buyOrderNumber++;
                     [buyOrderStorehouse fileDecreasingP: price
                                                andN: (float) number
                                usingAsNumberOfRows: buyOrderNumber];
                     if(printing==1)
                     [buyOrderStorehouse printNRows: buyOrderNumber];
                    }
               }
   }
   else
   { 
// the agent is selling at min price '-price'
   if(price<0) {if (buyOrderNumber>0 )
                    {executedPrice=[buyOrderStorehouse R: 0 C: 0];
                     currentMeanPrice+=executedPrice;
                     count++;
                     message(agentArrayIndex, number, -executedPrice);
                     message(agentArrayIndex,(int)[buyOrderStorehouse R: 0 C: 1],
                                                       executedPrice);
                     buyOrderNumber--;
                     [buyOrderStorehouse shiftRowsDown: buyOrderNumber];
                     if(printing==1)
                     [buyOrderStorehouse printNRows: buyOrderNumber];
                    }
                   
               else {
                     message(agentArrayIndex, number, 0.0);
			   if(printing==1)
                     printf("The book received a market order from agent #%3d but the order list has not orders \n",
          number);

                    }
               }

   // the agent is buying at max price 'price'
   if(price>0) {if (sellOrderNumber>0)
                    {executedPrice=[sellOrderStorehouse R: 0 C: 0];
                     currentMeanPrice+=executedPrice;
                     count++;
                     message(agentArrayIndex, number, executedPrice);
                     message(agentArrayIndex,(int)[sellOrderStorehouse R: 0 C: 1],
                                                      -executedPrice);
                     sellOrderNumber--;
                     [sellOrderStorehouse shiftRowsDown: sellOrderNumber];
                     if(printing==1)
                     [sellOrderStorehouse printNRows: sellOrderNumber];
                    }
                   
               else {
                     message(agentArrayIndex, number, 0.0);
			   if(printing==1)
                     printf("The book received a market order from agent #%3d but the order list has not orders \n",number);
                    }
               }

   }
   return self;
}
- (float) getSpread
{
  return [sellOrderStorehouse R: 0 C: 0]-[buyOrderStorehouse R: 0 C: 0];
}
- (float) getBuySellQuantitySpread
{
  return sellOrderNumber-buyOrderNumber;
}
- (float) getBuyFirstLastSpread
{
  return [buyOrderStorehouse R: 0 C: 0]-[buyOrderStorehouse R: buyOrderNumber-1 C: 0];
}
- (float) getSellFirstLastSpread
{
  return [sellOrderStorehouse R: sellOrderNumber-1 C: 0]-[sellOrderStorehouse R: 0 C: 0];
}
- (float) getPrice
{
  return executedPrice;
}

- (float) getMeanPrice
{
  return meanPrice;
}

- (float) getLaggedMeanPrice: (int) lag
{
  return [meanPriceHistory R: lag-1 C: 0];
}

- (float) getMeanPriceIndex
{
  return meanPrice/[meanPriceHistory R: nAheadForecasting C: 0];
}

- (int) getLocalHistory
{
  return getLocal(localHistory);
}

- (float) getSellOrderNumber
{
  return (float) sellOrderNumber;
}

- (float) getBuyOrderNumber
{
  return (float) buyOrderNumber;
}
- (float) getSellOrderNumberBeforeOpening
{
  return (float) sellOrderNumberBeforeOpening;
}

- (float) getBuyOrderNumberBeforeOpening
{
  return (float) buyOrderNumberBeforeOpening;
}
- (float) getPreviousClosingPrice
{
  return previousClosingPrice;
}

@end

