// LocallyImitatingAgent.m                                        

#import "LocallyImitatingAgent.h"

// static to avoid 'duplicate name' error (the function is defined also 
// in the .m files of other kinds of agent; the content anyway is different)
static float chooseBuySellSwitch(id theBook, float asymmetricBuySellProb)
{
  float buySellSwitch;

   buySellSwitch=0.5; // the standard random choice situation

   // Deciding of imitating the local history
   if([theBook getLocalHistory]>0) // locally buying decisions prevail
                                   // (buying prob = asymmetricaBuySellProb)
                                  buySellSwitch=asymmetricBuySellProb;
   if([theBook getLocalHistory]<0) // locally selling decisions prevail
                                   // (buying prob = 1 - asym...)
                                  buySellSwitch=1-asymmetricBuySellProb;

  return buySellSwitch;
}


@implementation LocallyImitatingAgent
 

  // our ruleMaster
- setRuleMaster: r
{
  ruleMaster=r;
  return self;
}

  // Acting before opening
- act0
{
   float lastPrice;
   int i;

     // clearing executedPrices
        executedPriceCount=0;
        for (i=0;i<maxOrderQuantity;i++) [executedPrices R:i C:0 setFrom:0];

   actingBeforeOpening=0;

   // imitating the market
   buySellSwitch=chooseBuySellSwitch(theBook, asymmetricBuySellProb);

   lastPrice=[theBook getPrice];

   // Asking the ruleMaster for a position (if the price is > 0 we are
   // buyer; if it is <0 we are seller; if it is =0 we do nothing
   // the ruleMaster uses in this case the probability of operating
   // before opening
   price=[ruleMaster getPriceBeforeOpening: lastPrice
                     withBuySellP: buySellSwitch];

   if (price!=0) actingBeforeOpening=1;

   // how many orders?
   if (maxOrderQuantity==1) iMax=1;
    else iMax=[uniformIntRand getIntegerWithMin: 1 withMax: maxOrderQuantity];
   for (i=1;i<=iMax;i++)
   // if price==0 theBook is not recording it
   [theBook setOrderBeforeOpeningFromAgent: number atPrice: price];

   return self;
}

  // acting in the market
- act1
{
   float lastPrice;
   int i;

   // the agent has operated before opening
   if (actingBeforeOpening==1) return self;

   // imitating the market
   buySellSwitch=chooseBuySellSwitch(theBook, asymmetricBuySellProb);

   lastPrice=[theBook getPrice];

   price=[ruleMaster getPrice: lastPrice
                     withBuySellP: buySellSwitch];


   // how many orders?
   if (maxOrderQuantity==1) iMax=1;
    else iMax=[uniformIntRand getIntegerWithMin: 1 withMax: maxOrderQuantity];
   for (i=1;i<=iMax;i++)
   [theBook setOrderFromAgent: number atPrice: price];

   return self;
}

@end






