//Matrix2.m (this version adds local methods to Matrix class of bp-ct)

#import "Matrix2.h"
#import <stdlib.h> // calloc

@implementation Matrix2

// methods are commented in Matrix2.h

- (int) getRows
{
  return rows;
}

- shiftRowsDown: (int) nr
{
  int i, j;

  // if we have only one agent (rows==1) or no row to move (nr==0) no
  // action is required
  if(rows==1 || nr==0) return self;

  // row 0 has to be overwritten; we are moving down the nr rows from
  // row 1 and row nr
  for (i=1;i<=nr;i++)
  for (j=0;j<cols;j++)
  m[(i-1)*cols+j]=m[i*cols+j];

  return self;
}

- pushRowsDown // from 0 to rows-1 (NB it works in reverse order if compared
               // with the shiftRowsDown above (which is like a 'pop')
{
  int i, j;

  // if we have only one row (rows==1) no action is required
  if(rows==1) return self;

  // row 0 has to be overwritten; we are moving down all the rows

  for (i=[self getRows]-1;i>0;i--)
  for (j=0;j<cols;j++)
  m[i*cols+j]=m[(i-1)*cols+j];

  return self;
}

- fileIncreasingP: (float) p andN: (float) n usingAsNumberOfRows: (int) nr
{
  int i,j,ii;
  // this is the unique order
  if (nr==1)           {
                        m[0]=p;
                        m[1]=n;
                        return self;
                       }

  // the order has to be inserted between two rows or as last row

  // identifying the row ii (from 0 to nr-2) which is the last to contain
  // a price <= the new one (see * below); the new price will be inserted
  // in row ii+1

  ii=-1;
  for (i=0;i<=nr-2;i++) if (m[i*cols]<=p) ii=i;

  // moving rows from ii+1 to nr-2 to be placed from ii+2 and nr-1 
  if(ii+1 != nr-1) // if the insertion is in the new last row, no shift is
  {                // necessary
  for (i=nr-1;i>ii+1;i--)
  for (j=0;j<cols;j++)
  m[i*cols+j]=m[(i-1)*cols+j];
  }

  // inserting the new line
  m[(ii+1)*cols]  =p;
  m[(ii+1)*cols+1]=n;

  return self;
}

- fileDecreasingP: (float) p andN: (float) n usingAsNumberOfRows: (int) nr
{
  int i,j,ii;
  // this is the unique order
  if (nr==1)           {
                        m[0]=p;
                        m[1]=n;
                        return self;
                       }

  // the order has to be inserted between two rows or as last row

  // identifying the row ii (from 0 to nr-2) which is the last to contain
  // a price >= the new one (see * below); the new price will be inserted
  // in row ii+1

  ii=-1;
  for (i=0;i<=nr-2;i++) if (m[i*cols]>=p) ii=i;

  // moving rows from ii+1 to nr-2 to be placed from ii+2 and nr-1 
  if(ii+1 != nr-1) // if the insertion is in the new last row, no shift is
  {                // necessary
  for (i=nr-1;i>ii+1;i--)
  for (j=0;j<cols;j++)
  m[i*cols+j]=m[(i-1)*cols+j];
  }

  // inserting the new line
  m[(ii+1)*cols]  =p;
  m[(ii+1)*cols+1]=n;

  return self;
}

- printNRows: (int) nr
{
  int i,j;

  if(nr==0){printf("Zero rows in Code=%1d matrix\n",code);return self;}

  printf("Rows in Code=%1d matrix\n",code);
  for (i=0;i<nr;i++)
   {for (j=0;j<cols;j++) printf("%7.4f ",m[i*cols+j]);
    printf("\n");
   }
  return self;
}

@end
                                     
// (*) if the new p is exactly equal to existing prices, it is
//     filed after the preceeding equal prices (if two or more orders are
//     equivalent, they are executed in order of time)




















