// ModelSwarm.h
#import <objectbase/Swarm.h>
#import <simtools.h>      // necessary to invoke ObjectLoader
#import <objectbase.h>    // needed by <ProbeMap> in ModelSwarm.m
#import <activity.h>
#import <collections.h>

  // agentForge step 2
#import "BasicSumAgent.h"
#import "RandomAgent.h"
#import "MarketImitatingAgent.h"
#import "LocallyImitatingAgent.h"
#import "StopLossAgent.h"
#import "ANNForecastAppAgent.h"
#import "ForecastingAgent.h"
#import "CurrentAgent.h"
#import "BPCTAgentA.h"
#import "BPCTAgentAInterface.h" // related to the agents used in our simulation
#import "BPCTAgentB.h"
#import "BPCTAgentBInterface.h" // related to the agents used in our simulation
#import "BPCTPriceRuleMaster.h"

#import "Book.h"
#import "RandomRuleMaster.h"
#import "StopLossRuleMaster.h"
#import "SimpleANNRuleMaster.h"
#import "SimpleANNRuleMaker.h"
#import "MatrixMult.h"
#import "VectorTransFunc.h"
#import "TransFunc.h"
#import "Quota.h"
#import <random.h> // to generate ad hoc private distributions to be used
                   // in SimpleANN and BPCT, to avoid interferences with random
                   // sequences used in determining agent behavior
#import "BPCTRuleMaster.h"
#import "BPCTRuleMaker.h"
#import "BPCTDataWarehouse.h"

@interface ModelSwarm: Swarm
{
  int dayNumber, maxOrderQuantity, meanPriceHistoryLength,
      localHistoryLength, stopLossInterval, checkingIfShortOrLong, printing,
      dataWindowLength, nAheadForecasting, forecastingTrainingSetLength,
      epochNumberInEachForecastingTrainingCycle, learningProcessEveryNDays,
      cleanForecastingANNEveryMgtemNDays;
  float asymmetricBuySellProb, agentProbToActBeforeOpening,probMarketOrder,
        minCorrectingCoeff, maxCorrectingCoeff,
        asymmetricRange, floorP, agentProbToActBelowFloorP,
        maxLossRate,
        aNNInactivityRange, aNNForecastAppAgentActDailyProb;

  // agentForge step 3
  int agentNumber, randomAgentNumber, marketImitatingAgentNumber,
    locallyImitatingAgentNumber, stopLossAgentNumber, 
    aNNForecastAppAgentNumber, bPCTAgentANumber, 
    bPCTAgentAEO_EP_0_Number, bPCTAgentAEO_EP_1_Number, 
    bPCTAgentAEO_EP_2_Number, bPCTAgentAEO_EP_3_Number,
                               bPCTAgentBNumber, 
    bPCTAgentBEO_EP_0_Number, bPCTAgentBEO_EP_1_Number, 
    bPCTAgentBEO_EP_2_Number, bPCTAgentBEO_EP_3_Number;
  id <List> agentList, randomAgentList, marketImitatingAgentList,
            locallyImitatingAgentList, stopLossAgentList, aNNForecastAppAgentList,
            bPCTAgentAList, bPCTAgentBList;

  id <Array> bPCTAgentAArray;
  id <Index> bPCTAgentAArrayIndex;

  id <Array> bPCTAgentBArray;
  id <Index> bPCTAgentBArrayIndex;

  id <Array> agentArray;
  id <Index> agentArrayIndex;
  id <ListShuffler> listShuffler;

  id <ActionGroup> modelActions1, modelActions2, modelActions3;
  id <Schedule> modelSchedule;

  // agentForge step 3r

  // see above about private distributions
  id <SimpleRandomGenerator> myGenerator; 
  id <UniformDoubleDist> myUniformDblRand;
  id <UniformIntegerDist> myUniformIntRand; 
  id <SimpleRandomGenerator> myGenerator2; 
  id <UniformDoubleDist> myUniformDblRand2;
  id <UniformIntegerDist> myUniformIntRand2; 
  id <SimpleRandomGenerator> myGenerator3; 
  id <UniformDoubleDist> myUniformDblRand3;
  id <UniformIntegerDist> myUniformIntRand3; 

  Book * theBook;
  RandomRuleMaster * randomRuleMaster;
  StopLossRuleMaster * stopLossRuleMaster;

  CurrentAgent * theCurrentAgent;
  ForecastingAgent * forecastingAgent;

  SimpleANNRuleMaster * simpleANNRuleMaster;
  SimpleANNRuleMaker * simpleANNRuleMaker;
  MatrixMult * matrixMult;
  VectorTransFunc * vectorTransFunc;
  TransFunc * transFunc;
  Quota * quota;


  // BPCT

  // agentForge step 3b
  int epochNumberInEachBPCTTrainingCycle;
  float agentAEO_EPDelta, agentBEO_EPDelta;

  // we are creating independent RuleMaster/Maker for each type of 
  // BPCT agent, to have separated random distributions also in weight
  // generation
  BPCTRuleMaster * bPCTRuleMasterA, * bPCTRuleMasterB;
  BPCTRuleMaker * bPCTRuleMakerA, * bPCTRuleMakerB;
  BPCTPriceRuleMaster * bPCTPriceRuleMasterA, * bPCTPriceRuleMasterB; 

  // agentForge step 3c
  int bPCTAgentAInputNodeNumber, bPCTAgentAHiddenNodeNumber,
      bPCTAgentAOutputNodeNumber, 
      bPCTAgentBInputNodeNumber, bPCTAgentBHiddenNodeNumber,
      bPCTAgentBOutputNodeNumber,

      bPCTPatternNumberInVerificationSet,
      bPCTPatternNumberInTrainingSet, bPCTAgentsAreDisplayingData,
      usingRandomOrderInBPCTLearning,                           
      longTermLearningInBPCT_OnlyWithCompleteTrainingSet,
      useOutputsAsTargetsInBPCT_RelearningScheme;
  float bPCTWeightRange, bPCTEps, bPCTAlpha;

}


+ createBegin: aZone;
- createEnd;
- buildObjects;
- buildActions;
- activateIn: swarmContext;

- increaseCurrentDayNumber;
- (int) getCurrentDay;
- getAgentArrayIndex;
- getBPCTAgentAArrayIndex;
- getBPCTAgentBArrayIndex;
- getBook;

  // agentForge step 4
- (int) getBPCTAgentANumber;
- getBPCTAgentAArrayIndex;
- (int) getBPCTAgentBNumber;
- getBPCTAgentBArrayIndex;
- getAgentList;
- getRandomAgentList;
- getMarketImitatingAgentList;
- getLocallyImitatingAgentList;
- getStopLossAgentList;
- getANNForecastAppAgentList;
- getBPCTAgentAList;
- getBPCTAgentBList;

- getForecastingAgent;
- openProbeTo: (int) ag;

@end
