// ObserverSwarm.m

#import "ObserverSwarm.h"
#import <activity.h>
#import <simtoolsgui.h>

@implementation ObserverSwarm

+ createBegin: aZone
{
  ObserverSwarm *obj;
  id <ProbeMap> probeMap;

  // Superclass createBegin to allocate ourselves.

  obj = [super createBegin: aZone];

  // Fill in the relevant parameters.

  obj->displayFrequency          = 1;

  obj->stopAtDayNumber           = 21; // to stop the program (if != 0)
  obj->displayPreviousDayMean    = 1;
  obj->savePriceData             = 1;
  obj->showBookGraph             = 1;
  obj->saveBookData              = 1;
  
  obj->showSpreadGraph                 = 1;
  obj->saveSpreadData                  = 1;
  obj->showBuySellQuantitySpreadGraph  = 1;
  obj->saveBuySellQuantitySpreadData   = 1;
  obj->showBuySellFirstLastSpreadGraph = 1;
  obj->saveBuySellFirstLastSpreadData  = 1;
  obj->showBuySellOrderNumberBeforeOpening =1;

  obj->showAgentWealthGraph      = 0;
  obj->saveAgentWealthData       = 0;
  obj->showForecastingAgentGraph = 0;
  obj->saveForecastingData       = 0;

  //agentForge step 11b
  obj->numberOfTheBPCTAgentA_ToBeObservedDirectly = 0;
  obj->saveBPCTAgentAData = 0;
  obj->numberOfTheBPCTAgentB_ToBeObservedDirectly = 0;
  obj->saveBPCTAgentBData = 0;

  // to build a customized probe map
  // without a probe map, the default is to show all variables and messages
  // here we choose to customize the appearance of the probe, to give a nicer
  // interface

  probeMap = [EmptyProbeMap createBegin: aZone];
  [probeMap setProbedClass: [self class]];
  probeMap = [probeMap createEnd];

  // add variables to be probed

  [probeMap addProbe: [probeLibrary getProbeForVariable: 
            "displayFrequency"           inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable:
            "stopAtDayNumber"            inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable:
            "displayPreviousDayMean"     inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable:
            "savePriceData"              inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable:
            "showBookGraph"              inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable:
            "saveBookData"               inClass: [self class]]];
  
  [probeMap addProbe: [probeLibrary getProbeForVariable:
            "showSpreadGraph"                 inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable:
            "saveSpreadData"                  inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable:
            "showBuySellQuantitySpreadGraph"  inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable:
            "saveBuySellQuantitySpreadData"   inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable:
            "showBuySellFirstLastSpreadGraph" inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable:
            "saveBuySellFirstLastSpreadData"  inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable:
            "showBuySellOrderNumberBeforeOpening"  inClass: [self class]]];        
  
  [probeMap addProbe: [probeLibrary getProbeForVariable:
            "showAgentWealthGraph"       inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable:
            "saveAgentWealthData"        inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable:
            "showForecastingAgentGraph"  inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable:
            "saveForecastingData"        inClass: [self class]]];

  //agentForge step 11c
  [probeMap addProbe: [probeLibrary getProbeForVariable:
            "numberOfTheBPCTAgentA_ToBeObservedDirectly" 
                                         inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable:
            "saveBPCTAgentAData"         inClass: [self class]]];

  [probeMap addProbe: [probeLibrary getProbeForVariable:
            "numberOfTheBPCTAgentB_ToBeObservedDirectly" 
                                         inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable:
            "saveBPCTAgentBData"         inClass: [self class]]];

  // set our custom probeMap into the probeLibrary as the default probe for
  // the ObserverSwarm class

  [probeLibrary setProbeMap: probeMap For: [self class]];

  return obj;
}

- createEnd
{
  return [super createEnd];
}


- buildObjects
{
  [super buildObjects];

  modelSwarm = [ModelSwarm create: self];
  

  // to create probe objects on the model and the observer (self, here)
  // ARCHIVED to allow the "Save" button to operate

  CREATE_ARCHIVED_PROBE_DISPLAY (modelSwarm);
  CREATE_ARCHIVED_PROBE_DISPLAY (self);

  // we pause here to allow the parameters to be changed.

  [controlPanel setStateStopped];


  // The system will wait until the user hits "Start" or "Next"
  // on the control panel

  [modelSwarm buildObjects];

  // checking the consistence of the displayFrequency with the agentNumber:
  // for display reasons it is necessary to update the graphic widgets
  // after all agent actions (mainly the BPCT ones, that calculate their 
  // targets at the end of a day and their inputs at the beginning of the day)
  // so the displays are updated at displayFrequency-1
  // if displayFrequency=1, in the agentNumber steps of a day we have the
  // new BPCT input and output updated at the first step and the BPCT target
  // updated at the last
  // if we adopt a displayFrequency multiple of angentNumber we have the
  // apparent coincidence of these values (##)

  if(displayFrequency !=1 && displayFrequency % 
     [[modelSwarm getAgentList] getCount] != 0)
    {
      printf("displayFrequency must be 1 or multiple of agentNumber.\n");
      exit(0);
    }

  theBook=[modelSwarm getBook];
  forecastingAgent=[modelSwarm getForecastingAgent];

  // Observer display objects.

  // The current price graph
  priceGraph = [EZGraph createBegin: self];
  SET_WINDOW_GEOMETRY_RECORD_NAME (priceGraph); // to allow "Save"

  if(savePriceData==1)[priceGraph setFileOutput: (BOOL) 1];
                      // to send the data also to a file

  [priceGraph setTitle: "Current price"];
  [priceGraph setAxisLabelsX: "Ticks x days." Y: "Price"];
  priceGraph = [priceGraph createEnd];
  // that above is the old way of creating an EZgraph; now a unique
  // methos exists

  // pay attention: the lines below cannot be placed before createEnd
  [priceGraph createSequence: "Current price" withFeedFrom: theBook
                                              andSelector: M(getPrice)];
  if(displayPreviousDayMean==1)
  [priceGraph createSequence: "Day-1 mean p." withFeedFrom: theBook
                                              andSelector: M(getMeanPrice)];
  //Buy Sell Spread graph
  if (showSpreadGraph==1)
  {
  spreadGraph = [EZGraph createBegin: self];
  SET_WINDOW_GEOMETRY_RECORD_NAME (spreadGraph); // to allow "Save"

  if(saveSpreadData==1)[spreadGraph setFileOutput: (BOOL) 1];
                      // to send the data also to a file

  [spreadGraph setTitle: "Current spread"];
  [spreadGraph setAxisLabelsX: "Ticks x days." Y: "spread"];
  spreadGraph = [spreadGraph createEnd];
  // that above is the old way of creating an EZgraph; now a unique
  // methos exists

  // pay attention: the lines below cannot be placed before createEnd
  [spreadGraph createSequence: "Current spread" withFeedFrom: theBook
                                              andSelector: M(getSpread)];
  }
  //Buy Sell Quantity Spread graph
  if (showBuySellQuantitySpreadGraph==1)
  {
  buysellQuantityGraph = [EZGraph createBegin: self];
  SET_WINDOW_GEOMETRY_RECORD_NAME (buysellQuantityGraph); // to allow "Save"

  if(saveBuySellQuantitySpreadData==1)[buysellQuantityGraph setFileOutput: (BOOL) 1];
                      // to send the data also to a file

  [buysellQuantityGraph setTitle: "Current quantity spread of buy and sell orders"];
  [buysellQuantityGraph setAxisLabelsX: "Ticks x days." Y: "Quantity Spread"];
  buysellQuantityGraph = [buysellQuantityGraph createEnd];
  // that above is the old way of creating an EZgraph; now a unique
  // methos exists

  // pay attention: the lines below cannot be placed before createEnd
  [buysellQuantityGraph createSequence: "Current quantity spread of buy and sell orders" withFeedFrom: theBook
                                              andSelector: M(getBuySellQuantitySpread)];
  }
  
  
  // The difference between first and last order of buy and sell lists graph

  if (showBuySellFirstLastSpreadGraph==1)
  {
  buysellFirstLastGraph = [EZGraph createBegin: self];
  SET_WINDOW_GEOMETRY_RECORD_NAME (buysellFirstLastGraph); // to allow "Save"

  if(saveBuySellFirstLastSpreadData==1)[buysellFirstLastGraph setFileOutput: (BOOL) 1];
                     // to send the data also to a file

  [buysellFirstLastGraph setTitle: "first-last difference prices of buy and sell order lists"];
  [buysellFirstLastGraph setAxisLabelsX: "Ticks x days." Y: "Sell and Buy differences"];
  buysellFirstLastGraph = [buysellFirstLastGraph createEnd];
  // that above is the old way of creating an EZgraph; now a unique
  // methos exists

  // pay attention: the lines below cannot be placed before createEnd

  [buysellFirstLastGraph createSequence: "Sell order list" withFeedFrom: theBook
                                        andSelector: M(getSellFirstLastSpread)];
  [buysellFirstLastGraph createSequence: "Buy order list"  withFeedFrom: theBook
                                        andSelector: M(getBuyFirstLastSpread)];
  }

  // The book graph

  if (showBookGraph==1)
  {
  bookGraph = [EZGraph createBegin: self];
  SET_WINDOW_GEOMETRY_RECORD_NAME (bookGraph); // to allow "Save"

  if(saveBookData==1)[bookGraph setFileOutput: (BOOL) 1];
                     // to send the data also to a file

  [bookGraph setTitle: "Book log"];
  [bookGraph setAxisLabelsX: "Ticks x days." Y: "Sell and Buy Orders in Log."];
  bookGraph = [bookGraph createEnd];
  // that above is the old way of creating an EZgraph; now a unique
  // methos exists

  // pay attention: the lines below cannot be placed before createEnd

  [bookGraph createSequence: "Sell orders" withFeedFrom: theBook
                                        andSelector: M(getSellOrderNumber)];
  [bookGraph createSequence: "Buy orders"  withFeedFrom: theBook
                                        andSelector: M(getBuyOrderNumber)];
  
   if(showBuySellOrderNumberBeforeOpening==1)
   {
   [bookGraph createSequence: "Sell orders before opening" withFeedFrom: theBook
                                        andSelector: M(getSellOrderNumberBeforeOpening)];
   [bookGraph createSequence: "Buy orders before opening"  withFeedFrom: theBook
                                        andSelector: M(getBuyOrderNumberBeforeOpening)];
   }  
  }

  // The agent's wealth graph

  if (showAgentWealthGraph==1)
  {
  agentWealthGraph = [EZGraph createBegin: self];
  SET_WINDOW_GEOMETRY_RECORD_NAME (agentWealthGraph); // to allow "Save"

  if(saveAgentWealthData==1)[agentWealthGraph setFileOutput: (BOOL) 1];
                      // to send the data also to a file

  [agentWealthGraph setTitle: "Agent's wealth"];
  [agentWealthGraph setAxisLabelsX: "Ticks x days."
                                     Y: "Wealth."];
  agentWealthGraph = [agentWealthGraph createEnd];
  // that above is the old way of creating an EZgraph; now a unique
  // methos exists

  // pay attention: the lines below cannot be placed before createEnd

  [agentWealthGraph createMinSequence: "MinWealth (all)"
                    withFeedFrom: [modelSwarm getAgentList]
                    andSelector: M(getWealthAtMeanDailyPrice)];
  [agentWealthGraph createAverageSequence: "MeanWealth (all)"
                    withFeedFrom: [modelSwarm getAgentList]
                    andSelector: M(getWealthAtMeanDailyPrice)];
  [agentWealthGraph createMaxSequence: "MaxWealth (all)"
                    withFeedFrom: [modelSwarm getAgentList]
                    andSelector: M(getWealthAtMeanDailyPrice)];
  [agentWealthGraph createMinSequence: "MinWealth (r.)"
                    withFeedFrom: [modelSwarm getRandomAgentList]
                    andSelector: M(getWealthAtMeanDailyPrice)];
  [agentWealthGraph createAverageSequence: "MeanWealth (r.)"
                    withFeedFrom: [modelSwarm getRandomAgentList]
                    andSelector: M(getWealthAtMeanDailyPrice)];
  [agentWealthGraph createMaxSequence: "MaxWealth (r.)"
                    withFeedFrom: [modelSwarm getRandomAgentList]
                    andSelector: M(getWealthAtMeanDailyPrice)];
  [agentWealthGraph createMinSequence: "MinWealth (m.i.)"
                    withFeedFrom: [modelSwarm getMarketImitatingAgentList]
                    andSelector: M(getWealthAtMeanDailyPrice)];
  [agentWealthGraph createAverageSequence: "MeanWealth (m.i.)"
                    withFeedFrom: [modelSwarm getMarketImitatingAgentList]
                    andSelector: M(getWealthAtMeanDailyPrice)];
  [agentWealthGraph createMaxSequence: "MaxWealth (m.i.)"
                    withFeedFrom: [modelSwarm getMarketImitatingAgentList]
                    andSelector: M(getWealthAtMeanDailyPrice)];
  [agentWealthGraph createMinSequence: "MinWealth (l.i.)"
                    withFeedFrom: [modelSwarm getLocallyImitatingAgentList]
                    andSelector: M(getWealthAtMeanDailyPrice)];
  [agentWealthGraph createAverageSequence: "MeanWealth (l.i.)"
                    withFeedFrom: [modelSwarm getLocallyImitatingAgentList]
                    andSelector: M(getWealthAtMeanDailyPrice)];
  [agentWealthGraph createMaxSequence: "MaxWealth (l.i.)"
                    withFeedFrom: [modelSwarm getLocallyImitatingAgentList]
                    andSelector: M(getWealthAtMeanDailyPrice)];
  [agentWealthGraph createMinSequence: "MinWealth (s.l.)"
                    withFeedFrom: [modelSwarm getStopLossAgentList]
                    andSelector: M(getWealthAtMeanDailyPrice)];
  [agentWealthGraph createAverageSequence: "MeanWealth (s.l.)"
                    withFeedFrom: [modelSwarm getStopLossAgentList]
                    andSelector: M(getWealthAtMeanDailyPrice)];
  [agentWealthGraph createMaxSequence: "MaxWealth (s.l.)"
                    withFeedFrom: [modelSwarm getStopLossAgentList]
                    andSelector: M(getWealthAtMeanDailyPrice)];
  [agentWealthGraph createMinSequence: "MinWealth (ann)"
                    withFeedFrom: [modelSwarm getANNForecastAppAgentList]
                    andSelector: M(getWealthAtMeanDailyPrice)];
  [agentWealthGraph createAverageSequence: "MeanWealth (ann)"
                    withFeedFrom: [modelSwarm getANNForecastAppAgentList]
                    andSelector: M(getWealthAtMeanDailyPrice)];
  [agentWealthGraph createMaxSequence: "MaxWealth (ann)"
                    withFeedFrom: [modelSwarm getANNForecastAppAgentList]
                    andSelector: M(getWealthAtMeanDailyPrice)];

  [agentWealthGraph createMinSequence: "MinWealth (bPCTA)"
                    withFeedFrom: [modelSwarm getBPCTAgentAList]
                    andSelector: M(getWealthAtMeanDailyPrice)];
  [agentWealthGraph createAverageSequence: "MeanWealth (bPCTA)"
                    withFeedFrom: [modelSwarm getBPCTAgentAList]
                    andSelector: M(getWealthAtMeanDailyPrice)];
  [agentWealthGraph createMaxSequence: "MaxWealth (bPCTA)"
                    withFeedFrom: [modelSwarm getBPCTAgentAList]
                    andSelector: M(getWealthAtMeanDailyPrice)];

  [agentWealthGraph createMinSequence: "MinWealth (bPCTB)"
                    withFeedFrom: [modelSwarm getBPCTAgentBList]
                    andSelector: M(getWealthAtMeanDailyPrice)];
  [agentWealthGraph createAverageSequence: "MeanWealth (bPCTB)"
                    withFeedFrom: [modelSwarm getBPCTAgentBList]
                    andSelector: M(getWealthAtMeanDailyPrice)];
  [agentWealthGraph createMaxSequence: "MaxWealth (bPCTB)"
                    withFeedFrom: [modelSwarm getBPCTAgentBList]
                    andSelector: M(getWealthAtMeanDailyPrice)];

  //agentForge step 11d (before this point)
  }
  // The forecastingAgent graph

  if (showForecastingAgentGraph==1)
  {
  forecastingAgentGraph = [EZGraph createBegin: self];
  SET_WINDOW_GEOMETRY_RECORD_NAME (forecastingAgentGraph); // to allow "Save"

  if(saveForecastingData==1)[forecastingAgentGraph setFileOutput: (BOOL) 1];
                                           // to send the data also to a file

  [forecastingAgentGraph setTitle: "Forecasting agent log"];
  [forecastingAgentGraph setAxisLabelsX: 
                         "Ticks x days." Y: "Estimates and errors."];
  forecastingAgentGraph = [forecastingAgentGraph createEnd];
  // that above is the old way of creating an EZgraph; now a unique
  // methos exists

  // pay attention: the lines below cannot be placed before createEnd

  [forecastingAgentGraph createSequence: "Day-1 mean p. (index)" 
			 withFeedFrom: theBook
                         andSelector: M(getMeanPriceIndex)];

  [forecastingAgentGraph createSequence: "Day-1 estimate (index)" 
			 withFeedFrom: forecastingAgent
                         andSelector: M(getPastEstimateIndex)];

  [forecastingAgentGraph createSequence: "Back propagation error" 
            withFeedFrom: forecastingAgent
            andSelector: M(getBackPropagationErrorInTrainingSet)];
  [forecastingAgentGraph createSequence: "Proportional error" 
            withFeedFrom: forecastingAgent
            andSelector: M(getProportionalErrorInTrainingSet)];
  }

  // --- BPCT ---

  // agent A

  if([modelSwarm getBPCTAgentANumber] == 0)
                         numberOfTheBPCTAgentA_ToBeObservedDirectly=0;

  // this may be 0 if we do not have any agent (above) or if we choose to
  // have no display
  if (numberOfTheBPCTAgentA_ToBeObservedDirectly > 0) 
                                // the value comes from probe
  {
  // to identify the address of the chosen agent

  [[modelSwarm getBPCTAgentAArrayIndex] setOffset:
                                numberOfTheBPCTAgentA_ToBeObservedDirectly-1];
  agentA = [[modelSwarm getBPCTAgentAArrayIndex] get];
  agentAInterface = [agentA getInterface];

   // agent A graph
   bPCTAgentA_Graph = [EZGraph createBegin: [self getZone]];
   SET_WINDOW_GEOMETRY_RECORD_NAME (bPCTAgentA_Graph); // to allow "Save"
   [bPCTAgentA_Graph setTitle: "BPCTAgent A data"];
   [bPCTAgentA_Graph setAxisLabelsX:
    "Ticks x days" Y: "Value"];
   bPCTAgentA_Graph = [bPCTAgentA_Graph createEnd];

   if(saveBPCTAgentAData==1)[bPCTAgentA_Graph setFileOutput: (BOOL) 1];
                                 // to send the data also to a file

   [bPCTAgentA_Graph createSequence: "meanPrice1_A"
                                 withFeedFrom: agentAInterface
                                 andSelector: M(getMeanPrice1)];

   [bPCTAgentA_Graph createSequence: "liquidityQuantity_out_A"
                                 withFeedFrom: agentAInterface
                                 andSelector: M(getLiquidityQuantity_out)];
   [bPCTAgentA_Graph createSequence: "liquidityQuantity_target_A"
                                 withFeedFrom: agentAInterface
                                 andSelector: M(getLiquidityQuantity_target)];
   [bPCTAgentA_Graph createSequence: "shareQuantity_out_A"
                                 withFeedFrom: agentAInterface
                                 andSelector: M(getShareQuantity_out)];
   [bPCTAgentA_Graph createSequence: "shareQuantity_target_A"
                                 withFeedFrom: agentAInterface
                                 andSelector: M(getShareQuantity_target)];
   [bPCTAgentA_Graph createSequence: "buySell_out_A"
                                 withFeedFrom: agentAInterface
                                 andSelector: M(getBuySell_out)];
   [bPCTAgentA_Graph createSequence: "buySell_target_A"
                                 withFeedFrom: agentAInterface
                                 andSelector: M(getBuySell_target)];

  }

  // agent B

  if([modelSwarm getBPCTAgentBNumber] == 0)
                         numberOfTheBPCTAgentB_ToBeObservedDirectly=0;

  // this may be 0 if we do not have any agent (above) or if we choose to
  // have no display
  if (numberOfTheBPCTAgentB_ToBeObservedDirectly > 0) 
                                // the value comes from probe
  {
  // to identify the address of the chosen agent

  [[modelSwarm getBPCTAgentBArrayIndex] setOffset:
                                numberOfTheBPCTAgentB_ToBeObservedDirectly-1];
  agentB = [[modelSwarm getBPCTAgentBArrayIndex] get];
  agentBInterface = [agentB getInterface];

   // agent B graph
   bPCTAgentB_Graph = [EZGraph createBegin: [self getZone]];
   SET_WINDOW_GEOMETRY_RECORD_NAME (bPCTAgentB_Graph); // to allow "Save"
   [bPCTAgentB_Graph setTitle: "BPCTAgent B data"];
   [bPCTAgentB_Graph setAxisLabelsX:
    "Ticks x days" Y: "Value"];
   bPCTAgentB_Graph = [bPCTAgentB_Graph createEnd];

   if(saveBPCTAgentBData==1)[bPCTAgentB_Graph setFileOutput: (BOOL) 1];
                                 // to send the data also to a file

   [bPCTAgentB_Graph createSequence: "meanPrice1_B"
                                 withFeedFrom: agentBInterface
                                 andSelector: M(getMeanPrice1)];

   [bPCTAgentB_Graph createSequence: "liquidityQuantity_out_B"
                                 withFeedFrom: agentBInterface
                                 andSelector: M(getLiquidityQuantity_out)];
   [bPCTAgentB_Graph createSequence: "liquidityQuantity_target_B"
                                 withFeedFrom: agentBInterface
                                 andSelector: M(getLiquidityQuantity_target)];
   [bPCTAgentB_Graph createSequence: "shareQuantity_out_B"
                                 withFeedFrom: agentBInterface
                                 andSelector: M(getShareQuantity_out)];
   [bPCTAgentB_Graph createSequence: "shareQuantity_target_B"
                                 withFeedFrom: agentBInterface
                                 andSelector: M(getShareQuantity_target)];
   [bPCTAgentB_Graph createSequence: "closingPriceWealth_out_B"
                                 withFeedFrom: agentBInterface
                                 andSelector: M(getClosingPriceWealth_out)];
   [bPCTAgentB_Graph createSequence: "closingPriceWealth_target_B"
                                 withFeedFrom: agentBInterface
                                 andSelector: M(getClosingPriceWealth_target)];
   [bPCTAgentB_Graph createSequence: "forecastedPriceWealth_out_B"
                                 withFeedFrom: agentBInterface
                                 andSelector: M(getForecastedPriceWealth_out)];
   [bPCTAgentB_Graph createSequence: "forecastedPriceWealth_target_B"
                                 withFeedFrom: agentBInterface
                                 andSelector: M(getForecastedPriceWealth_target)];
  [bPCTAgentB_Graph createSequence: "buySell_out_B"
                                 withFeedFrom: agentBInterface
                                 andSelector: M(getBuySell_out)];
   [bPCTAgentB_Graph createSequence: "buySell_target_B"
                                 withFeedFrom: agentBInterface
                                 andSelector: M(getBuySell_target)];

  }


  // agentForge step 11 (before this point, add ...)

  // to check (pressing only once the next or the start button) the initial 
  //settings, uncomment the following line
  //  [controlPanel setStateStopped];

  return self;
}

- buildActions
{
  [super buildActions];

  // model schedule.

  [modelSwarm buildActions];

  //agentForge step 11e
  bPCTPatternNumberInVerificationSet=0;
  if(agentA != nil)bPCTPatternNumberInVerificationSet=
                             [agentA getBPCTPatternNumberInVerificationSet];
  if(agentB != nil)bPCTPatternNumberInVerificationSet=
                             [agentB getBPCTPatternNumberInVerificationSet];
  // this variable is used below for a consistence check, as we need
  // here to have a verification set on length 1 (the same valued is used also
  // by the other BPCT agent; we have to repeat the assign operation in case
  // of missing categories of agents)

  // ActionGroup for Observer display

  displayActions = [ActionGroup create: self];

  // to update probes
  [displayActions createActionTo: probeDisplayManager message: M(update)];
  [displayActions createActionTo: actionCache message: M(doTkEvents)];

  [displayActions createActionTo: priceGraph message: M(step)];
  
  if (showSpreadGraph==1)
  [displayActions createActionTo: spreadGraph  message: M(step)];
  if (showBuySellQuantitySpreadGraph==1)
  [displayActions createActionTo: buysellQuantityGraph  message: M(step)];
  if (showBuySellFirstLastSpreadGraph==1)
  [displayActions createActionTo: buysellFirstLastGraph  message: M(step)];
  
  if (showBookGraph==1)
  [displayActions createActionTo: bookGraph  message: M(step)];
  if (showAgentWealthGraph==1)
  [displayActions createActionTo: agentWealthGraph  message: M(step)];
  if (showForecastingAgentGraph==1)
  [displayActions createActionTo: forecastingAgentGraph  message: M(step)];

  [displayActions createActionTo: self message: M(checkToStop)];

  // Display schedule. Note that the repeat interval is set by our
  // own Swarm data structure. The display is frequently the slowest part of a
  // simulation, when it is redraw less frequently things are faster

  displaySchedule = [Schedule createBegin: self];
  // we can use here a display frequency lower (e.g.1) than the number of steps
  // in the model (step number = agentNumber) to display the prices of each
  // step-tick
  [displaySchedule setRepeatInterval: displayFrequency];
  displaySchedule = [displaySchedule createEnd];

  [displaySchedule at: displayFrequency-1 createAction: displayActions];
                                              // see ## comment above

  //agentForge step 11f
  // BPCT

  if (numberOfTheBPCTAgentA_ToBeObservedDirectly > 0)
  // the following if condition is related to this CT use of the program, to
  // avoid nonsens
   if (bPCTPatternNumberInVerificationSet == -1)
     [displaySchedule at: displayFrequency-1  // see ## comment above 
                        createActionTo: bPCTAgentA_Graph message: M(step)];

  if (numberOfTheBPCTAgentB_ToBeObservedDirectly > 0)
  // the following if condition is related to this CT use of the program, to
  // avoid nonsens
   if (bPCTPatternNumberInVerificationSet == -1)
     [displaySchedule at: displayFrequency-1  // see ## comment above 
                        createActionTo: bPCTAgentB_Graph message: M(step)];

  return self;
}

- activateIn: swarmContext
{
// activateIn: - to activate the schedules to make them ready to run.

  [super activateIn: swarmContext];

  [modelSwarm activateIn: self];

  [displaySchedule activateIn: self];

  return [self getSwarmActivity];
}

  // to check for the stopping conditions
- checkToStop
{

  // if stopAtDayNumber is left to 0, the program will never 
  // this stop occurs after the first tick of the time in modelSwarm,
  // but this fact does not affect the results in the observerSwarm
  // being the stop performed before the graph update

  if (stopAtDayNumber !=0 &&
      stopAtDayNumber <= [modelSwarm getCurrentDay])
  {
      printf("Stopping at day number %4d\n",
                                [modelSwarm getCurrentDay]);
                                        
      // we can restart by pressing "Start" or "Next", but the simulation
      // will run for displayFrequency ticks and then it will stop again

      [controlPanel setStateStopped];
  }

  return self;
}



@end


